﻿--=======================================================================================================
--  GOLD NUGGET FINDER SCRIPT
-- 
-- Author:		akuenzi (akuenzi@gmail.com)
-- Date:		2/27/2019
-- Purpose:		Gives the player a path to the map's closest remaining gold nugget.
--=======================================================================================================


NuggetFinder = {};

function NuggetFinder:loadMap(name)
	self.getNuggetInRange = NuggetFinder.getNuggetInRange;
	
	self.firstRun = true;										-- Stuff to activate in the script just once.
	self.nuggetToggleActive = false;							-- Whether or not the script can activate.  No nuggets means no activation.
	self.nuggetToggle = false;									-- Player toggle to turn on nugget finding feature.
	self.distance = nil;										-- Distance between player and nugget.
	self.playerFaceNuggetToggle = false							-- Toggle to reset player to face towards nugget.
	self.currentNugget = nil									-- Id of current nugget located.
	
end;

function NuggetFinder:deleteMap()
end;

function NuggetFinder:mouseEvent(posX, posY, isDown, isUp, button)
end;

function NuggetFinder:keyEvent(unicode, sym, modifier, isDown) 
end;

function NuggetFinder:update(dt)
	
	if self.firstRun then
		if g_currentMission.collectableGoldNuggetsObject ~= nil then
			local nuggets = g_currentMission.collectableGoldNuggetsObject.goldNuggets
			for _, nugget in pairs(nuggets) do
				if nugget.goldNuggetTriggerId ~= 0 then
					-- We apparently have gold nuggets to find in the map.  Turn on the script.
					self.nuggetToggleActive = true;
					break
				end;
			end;
		else
			print("Gold Nugget Finder mod did not activate -- there do not appear to be any gold nuggets in this map.")
		end;
		self.firstRun = false
	end;
	
	-- If the script is active and no gui is active at the moment...
	if self.nuggetToggleActive and not g_gui:getIsGuiVisible() then
	
		-- Input binding to turn on/off the nugget finder.
		if InputBinding.hasEvent(InputBinding.NuggetFind) then
			self.nuggetToggle = not self.nuggetToggle;
			self.playerFaceNuggetToggle = false
			self.currentNugget = nil
		end;
		
		-- If the script is turned on by the player...
		if self.nuggetToggle then

			-- Is the player walking or driving?
			local player
			if g_currentMission.player.isEntered and g_currentMission.player.isControlled then
				-- Player is walking.
				player = g_currentMission.player.rootNode
			else
				-- Player is driving something.
				for i = 1, table.getn(g_currentMission.steerables) do
					if g_currentMission.steerables[i].isEntered then
						player = g_currentMission.steerables[i].rootNode;
						break
					end;
				end;
			end;
		
			-- Find the nugget closest to the player.
			local nearestNugget = self:getNuggetInRange(player);
			-- If we have a near nugget, then draw a line to it so the player knows where to walk/drive.
			if nearestNugget ~= nil then

				-- Get player and nugget position.
				local px, py, pz = getWorldTranslation(player);
				local vx, vy, vz = getWorldTranslation(nearestNugget);
				
				-- Raise y-values to get some elevation above map surface.
				py = py + 0.5
				vy = vy + 0.5
				
				-- Calculate world direction to nugget from player position, and reorient player to face nugget.
				if self.playerFaceNuggetToggle and (g_currentMission.player.isEntered and g_currentMission.player.isControlled) then
					local direction = math.atan2(px-vx, pz-vz);
					g_currentMission.player:setRotation(g_currentMission.player.rotX, direction);
					self.playerFaceNuggetToggle = false
				end;

				-- Update distance calculation between player and nugget.
				self.distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
				
				-- Render a line to the nugget from the player position.
				-- Line starts green (0,1,0) at player and ends up red (1,0,0) at nugget.
				drawDebugLine(px,py,pz, 0,1,0, vx,vy,vz, 1,0,0);
				--drawDebugLine(float x0, float y0, float z0, float r0, float g0, float b0, float x1, float y1, float z1, float r1, float g1, float b1)
			else
				-- We've found all the nuggets available.  Time to turn off script.
				self.nuggetToggleActive = false;
				self.distance = nil;
			end;
		end;
		
		-- Show help menu stuff.
		if self.nuggetToggle then
			local numNuggets = #g_currentMission.collectableGoldNuggetsObject.goldNuggets - g_currentMission.missionStats:getTotalValue("foundGoldNuggetCount")
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("input_NuggetFind"), g_i18n:getText("On")), InputBinding.NuggetFind, nil, GS_PRIO_LOW);
			if self.distance ~= nil then
				g_currentMission:addExtraPrintText(string.format(g_i18n:getText("distance"), tostring(numNuggets), tostring(Utils.round(self.distance,0))));
			end;
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("input_NuggetFind"), g_i18n:getText("Off")), InputBinding.NuggetFind, nil, GS_PRIO_LOW);
		end;
	end;
	
end;

function NuggetFinder:draw()
end;

function NuggetFinder:getNuggetInRange(player)
			
	-- Find the player's position.
	local px, py, pz = getWorldTranslation(player);

	local nearestNugget, distance = nil, nil;
	-- Parse nugget table to find one that is available.
	local nuggets = g_currentMission.collectableGoldNuggetsObject.goldNuggets;
	for _, nugget in pairs(nuggets) do
		if nugget.goldNuggetTriggerId ~= 0 then
			-- Get location of the found nugget.
			local vx, vy, vz = getWorldTranslation(nugget.goldNuggetId);
			-- Calculate distance from player to found nugget.
			local dist = Utils.vector3Length(px-vx, py-vy, pz-vz);
			-- Find the shortest distance between a found nugget and the player.
			if distance == nil then
				distance = dist;
				nearestNugget = nugget.goldNuggetId;
			else
				if dist < distance then
					distance = dist;
					nearestNugget = nugget.goldNuggetId;
				end;
			end;
		end;
	end;
	
	-- Check to see if we've found a new nugget.
	if nearestNugget ~= self.currentNugget then
		-- If so, then the player needs to be turned towards the new nugget.
		self.currentNugget = nearestNugget
		self.playerFaceNuggetToggle = true
	end;
	
	-- Send back the identity of the gold nugget closest to the player.
	return nearestNugget;
end;

addModEventListener(NuggetFinder);